/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * User form component <code>Mediator</code>.
 *
 * @requires org.puremvc.js.patterns.mediator.Mediator Mediator
 * @requires org.puremvc.js.patterns.observer.Notification Notification
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.vo.UserVO UserVO
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.UserProxy UserProxy
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.enum.DeptEnum DeptEnum
 * @requires org.puremvc.js.demos.objs.employeeadmin.view.components.UserForm UserForm
 *
 * @extends org.puremvc.js.patterns.mediator.Mediator Mediator
 *
 * @constructor
 */
var UserFormMediator = Objs("org.puremvc.js.demos.objs.employeeadmin.view.UserFormMediator",
	Mediator,
{	

	/**
	 * @private
	 *
	 * A shortcut to the application <code>RoleProxy</code> instance.
	 * 
	 * @type {RoleProxy}
	 */
	roleProxy: null,
		
	/**
	 * @private
	 *
	 * A shortcut to the application <code>UserProxy</code> instance.
	 * 
	 * @type {UserProxy}
	 */
	userProxy: null,

	/**
	 * @constructs
	 * @override
	 *
	 * Initialize a <code>UserFormMediator</code> instance.
	 * 
	 * @param {String} name
	 * 		Name for this <code>Mediator</code>.
	 *
	 * @param {UserForm} viewComponent
	 * 		The <code>UserForm</code> view Component this <code>Mediator</code>
	 * 		manage.
	 */
	initialize: function( name, viewComponent )
	{
		UserFormMediator.$super.initialize.call( this, name, viewComponent );
	
		this.registerListeners();

		this.userProxy = this.facade.retrieveProxy( ProxyNames.USER_PROXY );
		this.roleProxy = this.facade.retrieveProxy( ProxyNames.ROLE_PROXY );
	},
			
	/**
	 * @private
	 * 
	 * The <code>UserForm</code> view component this <code>Mediator</code> manage.
	 * 
	 * @return {UserForm}
	 */
	getUserForm : function()
	{
		return this.viewComponent;
	},

	/**
	 * Register event listeners for the UserForm component.
	 */
	registerListeners: function()
	{
		var userForm/*UserForm*/ = this.getUserForm();
		userForm.addEventListener( UserForm.ADD_USER, this.onAdd, this );
		userForm.addEventListener( UserForm.UPDATE_USER, this.onUpdate, this );
		userForm.addEventListener( UserForm.DELETE_USER, this.onDelete, this );
	},

	/**
	 * Unregister event listeners for the UserForm component.
	 */
	unregisterListeners: function()
	{
		var userForm/*UserForm*/ = this.getUserForm();
		userForm.removeEventListener( UserForm.ADD_USER, this.onAdd, this );
		userForm.removeEventListener( UserForm.UPDATE_USER, this.onUpdate, this );
		userForm.removeEventListener( UserForm.DELETE_USER, this.onDelete, this );
	},

	/**
	 * @private
	 * 
	 * Called when a user is added using the form.
	 * 
	 * @param {UiComponent.Event} event
	 * 		The dispatched event object.
	 */
	onAdd: function( event )
	{
		var user/*UserVO*/ = this.getUserForm().getUser();
		this.userProxy.addItem( user );
		this.updateUserRoles();
		this.sendNotification( NotificationNames.USER_ADDED, user );
	},
		
	/**
	 * @private
	 * 
	 * Called when a user is updated using the form.
	 */
	onUpdate: function()
	{
		var user/*UserVO*/ = this.getUserForm().getUser();
		this.userProxy.updateItem( user );
		this.updateUserRoles();
		this.sendNotification(  NotificationNames.USER_UPDATED, user );

	},
		
	/**
	 * @private
	 * 
	 * Called when a user is deleted.
	 */
	onDelete: function()
	{
		var user/*UserVO*/ = this.getUserForm().getUser();
		if( user == null )
			return;
		
		this.sendNotification
		(
			NotificationNames.DELETE_USER,
			user
		);
	},

	/*
	 * In the jQuery Mobile EmployeeAdmin demo we can't add/remove roles
	 * one by one, so we refresh the whole list on add/update operations.
	 */
	updateUserRoles: function()
	{
		var user/*UserVO*/ = this.getUserForm().getUser();
		var userRoles/*Array*/ = this.getUserForm().getUserRoles();
		var roleVO/*RoleVO*/ = new RoleVO();
		roleVO.uname = user.uname;
		roleVO.roles = userRoles;
		this.roleProxy.deleteItem( roleVO );
		this.roleProxy.addItem( roleVO );
	},
		
	/**
	 * @override
	 */
	listNotificationInterests: function()
	{
		return [
			NotificationNames.NEW_USER,
			NotificationNames.USER_SELECTED
		];
	},
		
	/**
	 * @override
	 */
	handleNotification: function( note )
	{
		var userForm/*UserForm*/ = this.getUserForm();
			
		var user/*UserVO*/;
		switch ( note.getName() )
		{
			case NotificationNames.NEW_USER:
				userForm.clearForm();
				userForm.setUser( note.getBody() );
				userForm.setUserRoles( [] );
				userForm.setMode( UserForm.MODE_ADD );
			break;
		
			case NotificationNames.USER_SELECTED:
				user = note.getBody();
		
				userForm.clearForm();
				userForm.setUser( user );
		
				var roles/*Array*/ = this.roleProxy.getUserRoles(user.uname);
				userForm.setUserRoles( roles );
		
				userForm.setMode( UserForm.MODE_EDIT );
			break;
		}
	},

	/**
	 * @override
	 *
	 * This will never be called during the demo but note that we well made the
	 * job of removing any listeners from the mediator and the component to
	 * make those instances ready for garbage collection.
	 */
	onRemove: function()
	{
		this.unregisterListeners();
		this.getUserForm().unbindListeners();
	}
});

/*
 * Constants
 */
UserFormMediator.ADD			= "add";
UserFormMediator.UPDATE			= "update";
UserFormMediator.CANCEL			= "cancel";

UserFormMediator.MODE_ADD		= "modeAdd";
UserFormMediator.MODE_EDIT		= "modeEdit";